/*
 * bdf2sfd 1.1.8
 * Copyright (c) 2019-2023, Frederic Cambus
 * https://github.com/fcambus/bdf2sfd
 *
 * Created:      2019-11-21
 * Last Updated: 2021-11-16
 *
 * bdf2sfd is released under the BSD 2-Clause license.
 * See LICENSE file for details.
 *
 * SPDX-License-Identifier: BSD-2-Clause
 */

#ifndef CONFIG_H
#define CONFIG_H

#define VERSION "bdf2sfd 1.1.8"

#endif /* CONFIG_H */
